// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file CouplingTimeInterface.h */

#ifndef AMDIS_COUPLINGTIMEINTERFACE_H
#define AMDIS_COUPLINGTIMEINTERFACE_H

#include "Flag.h"
#include "ProblemTimeInterface.h"
#include <vector>

namespace AMDiS {

  // ============================================================================
  // ===== class CouplingTimeInterface ==========================================
  // ============================================================================

  /**
   * \ingroup Problem 
   *
   * \brief
   */
  class CouplingTimeInterface : public ProblemTimeInterface
  {
  public:
    void addTimeInterface(ProblemTimeInterface *interface) {
      interfaces_.push_back(interface);
    };

    /** \brief
     * Executes all needed operations when the simulation time changes.
     */
    virtual void setTime(AdaptInfo *adaptInfo) {
      int i, size = static_cast<int>(interfaces_.size());
      for(i = 0; i < size; i++) {
	interfaces_[i]->setTime(adaptInfo);
      }
    };

    /** \brief
     * Called at the beginning of each timestep
     */
    virtual void initTimestep(AdaptInfo *adaptInfo) {
      int i, size = static_cast<int>(interfaces_.size());
      for(i = 0; i < size; i++) {
	interfaces_[i]->initTimestep(adaptInfo);
      }
    };

    /** \brief
     * Called at the end of each timestep.
     */
    virtual void closeTimestep(AdaptInfo *adaptInfo) {
      int i, size = static_cast<int>(interfaces_.size());
      for(i = 0; i < size; i++) {
	interfaces_[i]->closeTimestep(adaptInfo);
      }
    };

    /** \brief
     * Solves the initial problem.
     */
    virtual void solveInitialProblem(AdaptInfo *adaptInfo) {
      int i, size = static_cast<int>(interfaces_.size());
      for(i = 0; i < size; i++) {
	interfaces_[i]->solveInitialProblem(adaptInfo);
      }
    };  

    /** \brief
     * Solves the initial problem.
     */
    virtual void transferInitialSolution(AdaptInfo *adaptInfo) {
      int i, size = static_cast<int>(interfaces_.size());
      for(i = 0; i < size; i++) {
	interfaces_[i]->transferInitialSolution(adaptInfo);
      }
    };  

  protected:
    /** \brief
     * vector of coupled time interfaces
     */
    ::std::vector<ProblemTimeInterface*> interfaces_;
  };

}

#endif
